#!/bin/bash

#
#  Install script for Fixpoint Mathematical Library 
#  URL: http://code.google.com/p/fixpointlib/
#
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="100_board_olimex_p103"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------
  Dir=`pwd`
  if [ ! -e OK.Docs ]; then #{
    Get_Fails=""
    dir ../../Documentation/Boards

    getFile http://olimex.com/dev/pdf/ARM/ST/ STM32-P103.pdf Board_STM32-P103.pdf ../../Documentation/Boards/

    if [ "$Get_Fails" == "" ]; then
      touch OK.Docs
    else
      echo "$0 - ERROR: missing files: $Get_Fails"
    fi
  fi #}

  Name="${INSTALLPATH}" #{
  createExtensionMakefileHead ${Name} #{
  File="../extensions/makefile.${Name}" 
  cat <<END_OF_MAKEFILE >>$File 

# activate certain parts of source-code
COMPILE_OPTS += -DEXTENSION_$Name
COMPILE_OPTS += -DBOARD=$Name
BOARD = $Name

# define microcontroller used on board
STM32F10X_MD   = 1
COMPILE_OPTS += -DUC_DEVICE=STM32F103RBT6
COMPILE_OPTS += -DUC_PACKAGE_PINS=64

COMPILE_OPTS += -DRAM_SIZE_K=20
COMPILE_OPTS += -DRAM_START=0x20000000
COMPILE_OPTS += -DFLASH_SIZE_K=128
COMPILE_OPTS += -DFLASH_START=0x08000000

# set some well known port defines
COMPILE_OPTS += -DPB_LED1=GPIOC,12
COMPILE_OPTS += -DPB_LED2=GPIOC,12

COMPILE_OPTS += -DPB_SWITCH2=GPIOA,0    # press button switch 2 (named WKUP)

COMPILE_OPTS += -DTTC_USART1_TX=GPIOA,1   # RS232 connector 9-pin female TX-pin
COMPILE_OPTS += -DTTC_USART1_RX=GPIOA,2   # RS232 connector 9-pin female RX-pin
COMPILE_OPTS += -DTTC_USART1_RTS=GPIOA,1  # RS232 connector 9-pin female RTS-pin
COMPILE_OPTS += -DTTC_USART1_CTS=GPIOA,0  # RS232 connector 9-pin female CTS-pin
COMPILE_OPTS += -DTTC_USART1=USART2       # RS232 connector connected USART

COMPILE_OPTS += -DTTC_USART2_TX=GPIOA,9   # RS232 on 10-pin male header connector TX-Pin
COMPILE_OPTS += -DTTC_USART2_RX=GPIOA,10  # RS232 on 10-pin male header connector RX-Pin
COMPILE_OPTS += -DTTC_USART2=USART1       # RS232 on 10-pin male header connector connected USART

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh" #{
  createActivateScriptHead $File $0 "Protoboard P103 from Olimex -> http://olimex.com/dev/stm32-p103.html"
  cat <<END_OF_ACTIVATE >>$File #{

  rm 2>/dev/null \$Dir_ExtensionsActive/makefile.100_board_*
  
  # create links into extensions.active/
  createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET
  
  activate.200_cpu_stm32f10x.sh QUIET

END_OF_ACTIVATE
#}
  createActivateScriptTail $File $Name
  #}

#}
  echo "Installed successfully: $INSTALLPATH"

cd ..
rmdir 2>/dev/null $INSTALLPATH
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
