#!/bin/bash

#
#  Install script for Fixpoint Mathematical Library 
#  URL: http://code.google.com/p/fixpointlib/
#
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="100_board_olimex_lcd"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------
  Dir=`pwd`
  if [ ! -e $Dir/OK.Docs ]; then #{
    Get_Fails=""
    dir ../../Documentation/Boards
    cd ../../Documentation/Boards
    getFile http://olimex.com/dev/pdf/ARM/ST/ STM32-LCD.pdf Board_STM32-LCD.pdf
    getFile http://www.olimex.com/dev/pdf/PIC/ "FS-K320QVB-V1-01-3T%20(3).pdf" LCD_FS-K320QVB-V1-01-3T.pdf
    getFile http://olimex.com/dev/ stm32-lcd.html
    if [ "$Get_Fails" == "" ]; then
      touch $Dir/OK.Docs
    else
      echo "$0 - ERROR: missing files: $Get_Fails"
    fi
  fi #}
  cd "$Dir"

  Name="${INSTALLPATH}" #{
  createExtensionMakefileHead ${Name} #{
  File="../extensions/makefile.${Name}" 
  cat <<END_OF_MAKEFILE >>$File 

# activate certain parts of source-code
COMPILE_OPTS += -DEXTENSION_$Name
COMPILE_OPTS += -DBOARD=$Name
BOARD = $Name

# define microcontroller used on board
STM32F10X_HD   = 1
COMPILE_OPTS += -DUC_DEVICE=STM32F103ZE
COMPILE_OPTS += -DUC_PACKAGE_PINS=144

COMPILE_OPTS += -DRAM_SIZE_K=64
COMPILE_OPTS += -DRAM_START=0x20000000
COMPILE_OPTS += -DFLASH_SIZE_K=512
COMPILE_OPTS += -DFLASH_START=0x08000000

# set some well known port defines
COMPILE_OPTS += -DTTC_USART1_TX=GPIOA,9   # RS232 on 10-pin male header connector UEXT1 TX-Pin
COMPILE_OPTS += -DTTC_USART1_RX=GPIOA,10  # RS232 on 10-pin male header connector UEXT1 RX-Pin
COMPILE_OPTS += -DTTC_USART1=USART1       # RS232 on 10-pin male header connector UEXT1 connected USART

COMPILE_OPTS += -DTTC_USART2_TX=GPIOA,2   # RS232 on 10-pin male header connector UEXT2 TX-Pin
COMPILE_OPTS += -DTTC_USART2_RX=GPIOA,3   # RS232 on 10-pin male header connector UEXT2 RX-Pin
COMPILE_OPTS += -DTTC_USART2=USART2       # RS232 on 10-pin male header connector UEXT2 connected USART

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  File="../extensions/activate.${Name}.sh" #{
  createActivateScriptHead $File $0 "Protoboard LCD from Olimex -> http://olimex.com/dev/stm32-lcd.html"
  cat <<END_OF_ACTIVATE >>$File #{

  rm 2>/dev/null \$Dir_ExtensionsActive/makefile.100_board_*
  
  # create links into extensions.active/
  createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET
  
  activate.200_cpu_stm32f10x.sh QUIET
  activate.400_lcd_320x240_olimex.sh QUIET


END_OF_ACTIVATE
#}
  createActivateScriptTail $File $Name
  #}

#}
  echo "Installed successfully: $INSTALLPATH"

cd ..
rmdir 2>/dev/null $INSTALLPATH
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
