#!/bin/bash

Main="$1"

if [ "$Main" != "" ]; then
  if [ ! -e "mains/$Main" ]; then
    echo "$0 - ERROR: Cannot find main-file '$Main'!"
    echo "Available candidates: `ls -C mains/*.c`"
    exit 6
  fi
  rm 2>/dev/null main.c
  ln -s mains/$Main main.c
fi

if [ ! -e main.c ]; then
  echo "$0 - ERROR: No main.c selected!"
  echo "Available candidates: `ls -C mains/*.c`"
  exit 5
fi

cat <<END_OF_HEADER >extensions.active/makefile
#
#                               ToolChain_STM32
#
# Central makefile.
# This makefile was created by concatenating all makefile.* files in this folder.
# 
# This file has been automatically created by $0 on `date`.
# ALL CHANGES MADE TO THIS FILE WILL GET LOST!
#

END_OF_HEADER
find extensions.active/makefile\.* -exec cat {} >>extensions.active/makefile \;

ProjectFiles=`ls QtCreator/*.files`
for ProjectFile in $ProjectFiles; do
  echo "updating project file $ProjectFile .."
  ./_/updateProjectFiles.pl $ProjectFile ../ >${ProjectFile}.log || Error="1"
done
if [ "$Error" == "1" ]; then
  echo "Compilation canceld due to error"
  exit 10
fi

Log="compile.log"
OK=""
echo "compiling..."
ls -l main.c
make clean >/dev/null 2>&1
make all -j4 >$Log 2>&1 && OK="1"

if [ "$OK" == "1" ]; then
    echo "Compiled successfully:"
    ls -lh *.bin *.elf
    echo ""
else
    echo "ERROR occured during compilation!"
    ${PAGER:-less} $Log
    exit 10
fi
echo ""

